@@ -45,8 +45,14 @@ def plotTraj(r, figprefix="traj_", dpi=500):
     ax[0].plot(rpb[:,0], rpb[:,1], linewidth=0.2, label="rough beta", alpha=0.8)
     ax[0].plot(spa[:,0], spa[:,1], 'k-', linewidth=0.2, label="smooth alpha")
     ax[0].plot(spb[:,0], spb[:,1], 'k-', linewidth=0.2, label="smooth beta")
-    ax[0].plot(aRDP[:,0], aRDP[:,1], 'oc-', linewidth=0.2, markeredgewidth=0.4, fillstyle="none", markersize=2, label="RDP alpha", alpha=0.7)
-    ax[0].plot(bRDP[:,0], bRDP[:,1], 'oc-', linewidth=0.2, markeredgewidth=0.4, fillstyle="none", markersize=2, label="RDP beta", alpha=0.7)
+    ax[0].plot(
+        aRDP[:,0], aRDP[:,1], 'oc-', linewidth=0.2, markeredgewidth=0.4,
+        fillstyle="none", markersize=2, label="RDP alpha", alpha=0.7
+    )
+    ax[0].plot(
+        bRDP[:,0], bRDP[:,1], 'oc-', linewidth=0.2, markeredgewidth=0.4,
+        fillstyle="none", markersize=2, label="RDP beta", alpha=0.7
+    )
     ax[0].legend()
 
     ax[1].plot(vSteps, av, linewidth=0.2, label="alphaVel", alpha=0.4)
@@ -58,11 +64,15 @@ def plotTraj(r, figprefix="traj_", dpi=500):
     # plt.legend()
 
 
-    plt.savefig(figprefix+"robot_%s.png"%r.id, dpi=dpi)
+    plt.savefig(figprefix+"robot_%s.pdf"%r.id, dpi=dpi)
     plt.close()
 
 
-def plotOne(step, robotGrid=None, figname=None, isSequence=True, plotTargets=False, xlim=None, ylim=None, highlightRobot=None, returnax=False):
+def plotOne(
+    step, robotGrid=None, figname=None, isSequence=True,
+    plotTargets=False, plotRobotIDs=False, dpi=100,
+    xlim=None, ylim=None, highlightRobot=None, returnax=False
+):
     global rg
     if hasattr(step, "__len__"):
         fig = step[1]
@@ -74,11 +84,13 @@ def plotOne(step, robotGrid=None, figname=None, isSequence=True, plotTargets=Fal
         rg = robotGrid
     plt.figure(figsize=(10,10))
     ax = plt.gca()
+    ax.set_facecolor('lightgray')
     maxX = 0
     maxY = 0
     for robotID, robot in rg.robotDict.items():
         # account for xy calibration offsets in base
         # pos of robot in wok space
+        hatch = None
         wokBase = coordio.libcoordio.tangentToWok(
             [0,0,0],
             robot.basePos,
@@ -115,11 +127,14 @@ def plotOne(step, robotGrid=None, figname=None, isSequence=True, plotTargets=Fal
             betaX = betaPoint[0]
             betaY = betaPoint[1]
             onTarget = robot.score() == 0
-        plt.plot([basePos[0], alphaX], [basePos[1], alphaY], color='black', linewidth=2, alpha=0.5)
+        plt.plot(
+            [basePos[0], alphaX], [basePos[1], alphaY],
+            color='black', linewidth=2, alpha=0.5
+        )
 
         topCollideLine = LineString(
             [(alphaX, alphaY), (betaX, betaY)]
-        ).buffer(rg.collisionBuffer, cap_style=1)
+        ).buffer(robot.collisionBuffer, cap_style=1)
         topcolor = 'blue'
         edgecolor = 'black'
         if not robot.isAssigned():
@@ -129,23 +144,55 @@ def plotOne(step, robotGrid=None, figname=None, isSequence=True, plotTargets=Fal
         if not isSequence and rg.isCollided(robotID):
             # collision trumps not assigned
             topcolor = "red"
-        if highlightRobot == robotID:
-            topcolor = "orange"
+        if highlightRobot is not None:
+            if highlightRobot == robotID:
+                topcolor = "violet"
+                # edgecolor = "red"
+                hatch=None
+            elif hasattr(highlightRobot, "__len__") and robotID in highlightRobot:
+                topcolor = "violet"
+                # edgecolor = "red"
+                hatch = None
         if robot.isOffline:
             topcolor = "black"
-        patch = PolygonPatch(topCollideLine, fc=topcolor, ec=edgecolor, alpha=0.5, zorder=10)
+        patch = PolygonPatch(
+            topCollideLine, fc=topcolor, ec=edgecolor, hatch=hatch, alpha=0.5, zorder=10
+        )
         ax.add_patch(patch)
+        if plotRobotIDs:
+            strID = "P" + ("%i"%robot.id).zfill(4)
+            xm = (alphaX + betaX)/2
+            ym = (alphaY + betaY)/2
+            dx = alphaX-betaX
+            dy = alphaY-betaY
+            rot = numpy.degrees(numpy.arctan2(dy, dx))
+            if alphaX < betaX:
+                rot += 180
+            props = {"ha": "center", "va": "center", "fontsize": 2}
+            ax.text(xm, ym, strID, props, rotation=rot)
+
     for fiducialID, fiducial in rg.fiducialDict.items():
-        fPoint = Point(fiducial.xyzWok[0], fiducial.xyzWok[1]).buffer(fiducial.collisionBuffer) #, cap_style=1)
-        patch = PolygonPatch(fPoint, fc="cyan", ec="black", alpha=0.8, zorder=10)
+
+        fPoint = Point(
+            fiducial.xyzWok[0], fiducial.xyzWok[1]
+        ).buffer(fiducial.collisionBuffer) #, cap_style=1)
+
+        patch = PolygonPatch(fPoint, fc="aquamarine", ec="black", alpha=0.8, zorder=10)
+        ax.add_patch(patch)
+    for gfa in rg.gfaDict.values():
+
+        gLine = LineString(
+            [gfa.collisionSegWokXYZ[0], gfa.collisionSegWokXYZ[1]]
+        ).buffer(gfa.collisionBuffer, cap_style=1)
+        patch = PolygonPatch(gLine, fc="aquamarine", ec="black", alpha=0.5, zorder=10)
         ax.add_patch(patch)
 
     if xlim is not None:
         ax.set_xlim(xlim)
     if ylim is not None:
         ax.set_ylim(ylim)
-    ax.set_xlim([-maxX-30, maxX+30])
-    ax.set_ylim([-maxY-30, maxY+30])
+    ax.set_xlim([-maxX-50, maxX+50])
+    ax.set_ylim([-maxY-50, maxY+50])
     ax.set_aspect("equal")
 
     if returnax:
@@ -153,7 +200,7 @@ def plotOne(step, robotGrid=None, figname=None, isSequence=True, plotTargets=Fal
 
     if figname is None:
         figname = "step_%04d.png"%(fig)
-    plt.savefig(figname, dpi=100)
+    plt.savefig(figname, dpi=dpi)
     plt.close()
 
 def plotPaths(robotGrid, downsample=None, filename=None):
@@ -249,58 +296,6 @@ def hexFromDia(nDia, pitch=22.4, rotAngle=0):
     ind = numpy.lexsort((xAll, yAll))
     return xAll[ind], yAll[ind]
 
-# def robotGridFromFilledHex(stepSize=1, collisionBuffer=2, seed=0, rotAngle=0):
-#     gridFile = os.path.join(os.environ["KAIJU_DIR"], "etc", "fps_filledHex.txt")
-#     # Row   Col     X (mm)  Y (mm)          Assignment
-#     #
-#     #-13   0 -145.6000 -252.1866  BA
-#     #-13   1 -123.2000 -252.1866  BA
-#     bossXY = []
-#     baXY = []
-#     fiducialXY = []
-#     cos = numpy.cos(numpy.radians(rotAngle))
-#     sin = numpy.sin(numpy.radians(rotAngle))
-#     rotMat = numpy.array([
-#         [cos, sin],
-#         [-sin, cos]
-#     ])
-#     with open(gridFile, "r") as f:
-#         lines = f.readlines()
-#     for line in lines:
-#         line = line.strip()
-#         line = line.split("#")[0]
-#         if not line:
-#             continue
-#         row, col, x, y, fType = line.split()
-#         x = float(x)
-#         y = float(y)
-#         if rotAngle:
-#             x, y = numpy.dot([x,y], rotMat)
-#         coords = [x, y]
-#         if fType == "BA":
-#             baXY.append(coords)
-#         elif fType == "BOSS":
-#             bossXY.append(coords)
-#         else:
-#             fiducialXY.append(coords)
-
-
-#     epsilon = stepSize * 2.2
-
-#     rg = RobotGrid(stepSize, collisionBuffer, epsilon, seed)
-#     robotID = 0
-#     hasApogee = False
-#     for b in bossXY:
-#         rg.addRobot(robotID, str(robotID), b[0], b[1], hasApogee)
-#         robotID += 1
-#     hasApogee = True
-#     for ba in baXY:
-#         rg.addRobot(robotID, str(robotID), ba[0], ba[1], hasApogee)
-#         robotID += 1
-#     for fiducialID, fiducial in enumerate(fiducialXY):
-#         rg.addFiducial(fiducialID, fiducial[0], fiducial[1])
-#     rg.initGrid()
-#     return rg
 
 
 
