@@ -56,6 +56,9 @@ PYBIND11_MODULE(cKaiju, m) {
         .def_readwrite("alpha", &Robot::alpha, R"pbdoc(
             Robot's alpha position (degrees).
             )pbdoc")
+        .def_readwrite("fiducialWatch", &Robot::fiducialWatch)
+        .def_readwrite("greed", &Robot::greed)
+        .def_readwrite("phobia", &Robot::phobia)
         .def_readwrite("isOffline", &Robot::isOffline)
         .def_readwrite("lefthanded", &Robot::lefthanded)
         .def_readwrite("robotNeighbors", &Robot::robotNeighbors)
@@ -95,12 +98,18 @@ PYBIND11_MODULE(cKaiju, m) {
         .def_readwrite("assignedTargetID", &Robot::assignedTargetID)
         .def_readwrite("alphaPath", &Robot::alphaPath)
         .def_readwrite("betaPath", &Robot::betaPath)
+        .def_readwrite("alphaPathBuffered", &Robot::alphaPathBuffered)
+        .def_readwrite("betaPathBuffered", &Robot::betaPathBuffered)
         .def_readwrite("smoothedAlphaPath", &Robot::smoothedAlphaPath)
         .def_readwrite("smoothedBetaPath", &Robot::smoothedBetaPath)
         .def_readwrite("simplifiedAlphaPath", &Robot::simplifiedAlphaPath)
         .def_readwrite("simplifiedBetaPath", &Robot::simplifiedBetaPath)
         .def_readwrite("interpSimplifiedAlphaPath", &Robot::interpSimplifiedAlphaPath)
         .def_readwrite("interpSimplifiedBetaPath", &Robot::interpSimplifiedBetaPath)
+        .def_readwrite("interpVelocityAlphaPath", &Robot::interpVelocityAlphaPath)
+        .def_readwrite("interpVelocityBetaPath", &Robot::interpVelocityBetaPath)
+        .def_readwrite("interpAccelerationAlphaPath", &Robot::interpAccelerationAlphaPath)
+        .def_readwrite("interpAccelerationBetaPath", &Robot::interpAccelerationBetaPath)
         .def_readwrite("interpAlphaX", &Robot::interpAlphaX)
         .def_readwrite("interpAlphaY", &Robot::interpAlphaY)
         .def_readwrite("interpBetaX", &Robot::interpBetaX)
@@ -137,14 +146,15 @@ PYBIND11_MODULE(cKaiju, m) {
         .def_readwrite("algType", &RobotGrid::algType)
         .def_readwrite("seed", &RobotGrid::seed)
         .def_readwrite("robotDict", &RobotGrid::robotDict)
-        .def_readwrite("greed", &RobotGrid::greed)
-        .def_readwrite("phobia", &RobotGrid::phobia)
+        // .def_readwrite("greed", &RobotGrid::greed)
+        // .def_readwrite("phobia", &RobotGrid::phobia)
         .def_readwrite("angStep", &RobotGrid::angStep)
         // .def_readwrite("collisionBuffer", &RobotGrid::collisionBuffer)
         .def_readwrite("smoothCollisions", &RobotGrid::smoothCollisions)
         .def_readwrite("smoothCollidedRobots", &RobotGrid::smoothCollidedRobots)
         .def_readwrite("didFail", &RobotGrid::didFail)
         .def_readwrite("nSteps", &RobotGrid::nSteps)
+        .def_readwrite("mdp2iter", &RobotGrid::mdp2iter)
         // .def_readwrite("nStepsSmoothed", &RobotGrid::nStepsSmoothed)
         .def_readwrite("nRobots", &RobotGrid::nRobots)
         .def_readwrite("fiducialDict", &RobotGrid::fiducialDict)
@@ -182,7 +192,8 @@ PYBIND11_MODULE(cKaiju, m) {
         .def("shrinkCollisionBuffer", &RobotGrid::shrinkCollisionBuffer)
         .def("growCollisionBuffer", &RobotGrid::growCollisionBuffer)
         .def("pathGenGreedy", &RobotGrid::pathGenGreedy, "stopIfDeadlock"_a = false, "ignoreInitialCollisions"_a = false)
-        .def("pathGenMDP", &RobotGrid::pathGenMDP)
+        .def("pathGenMDP", &RobotGrid::pathGenMDP, "greed"_a, "phobia"_a, "ignoreInitialCollisions"_a = false)
+        .def("pathGenMDP2", &RobotGrid::pathGenMDP2, "greed"_a, "phobia"_a, "ignoreInitialCollisions"_a = false)
         .def("pathGenExplode", &RobotGrid::pathGenExplode)
         .def("pathGenExplodeOne", &RobotGrid::pathGenExplodeOne)
         .def("targetlessRobots", &RobotGrid::targetlessRobots)
